/*******************************************************************************
* File Name: PWM_Led_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PWM_Led.h"

static PWM_Led_backupStruct PWM_Led_backup;


/*******************************************************************************
* Function Name: PWM_Led_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_Led_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void PWM_Led_SaveConfig(void) 
{

    #if(!PWM_Led_UsingFixedFunction)
        #if(!PWM_Led_PWMModeIsCenterAligned)
            PWM_Led_backup.PWMPeriod = PWM_Led_ReadPeriod();
        #endif /* (!PWM_Led_PWMModeIsCenterAligned) */
        PWM_Led_backup.PWMUdb = PWM_Led_ReadCounter();
        #if (PWM_Led_UseStatus)
            PWM_Led_backup.InterruptMaskValue = PWM_Led_STATUS_MASK;
        #endif /* (PWM_Led_UseStatus) */

        #if(PWM_Led_DeadBandMode == PWM_Led__B_PWM__DBM_256_CLOCKS || \
            PWM_Led_DeadBandMode == PWM_Led__B_PWM__DBM_2_4_CLOCKS)
            PWM_Led_backup.PWMdeadBandValue = PWM_Led_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(PWM_Led_KillModeMinTime)
             PWM_Led_backup.PWMKillCounterPeriod = PWM_Led_ReadKillTime();
        #endif /* (PWM_Led_KillModeMinTime) */

        #if(PWM_Led_UseControl)
            PWM_Led_backup.PWMControlRegister = PWM_Led_ReadControlRegister();
        #endif /* (PWM_Led_UseControl) */
    #endif  /* (!PWM_Led_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PWM_Led_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_Led_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_Led_RestoreConfig(void) 
{
        #if(!PWM_Led_UsingFixedFunction)
            #if(!PWM_Led_PWMModeIsCenterAligned)
                PWM_Led_WritePeriod(PWM_Led_backup.PWMPeriod);
            #endif /* (!PWM_Led_PWMModeIsCenterAligned) */

            PWM_Led_WriteCounter(PWM_Led_backup.PWMUdb);

            #if (PWM_Led_UseStatus)
                PWM_Led_STATUS_MASK = PWM_Led_backup.InterruptMaskValue;
            #endif /* (PWM_Led_UseStatus) */

            #if(PWM_Led_DeadBandMode == PWM_Led__B_PWM__DBM_256_CLOCKS || \
                PWM_Led_DeadBandMode == PWM_Led__B_PWM__DBM_2_4_CLOCKS)
                PWM_Led_WriteDeadTime(PWM_Led_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(PWM_Led_KillModeMinTime)
                PWM_Led_WriteKillTime(PWM_Led_backup.PWMKillCounterPeriod);
            #endif /* (PWM_Led_KillModeMinTime) */

            #if(PWM_Led_UseControl)
                PWM_Led_WriteControlRegister(PWM_Led_backup.PWMControlRegister);
            #endif /* (PWM_Led_UseControl) */
        #endif  /* (!PWM_Led_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: PWM_Led_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_Led_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void PWM_Led_Sleep(void) 
{
    #if(PWM_Led_UseControl)
        if(PWM_Led_CTRL_ENABLE == (PWM_Led_CONTROL & PWM_Led_CTRL_ENABLE))
        {
            /*Component is enabled */
            PWM_Led_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            PWM_Led_backup.PWMEnableState = 0u;
        }
    #endif /* (PWM_Led_UseControl) */

    /* Stop component */
    PWM_Led_Stop();

    /* Save registers configuration */
    PWM_Led_SaveConfig();
}


/*******************************************************************************
* Function Name: PWM_Led_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_Led_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_Led_Wakeup(void) 
{
     /* Restore registers values */
    PWM_Led_RestoreConfig();

    if(PWM_Led_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        PWM_Led_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
