/**
*   \file main.c
*   
*   Project to test I2C slave device with DMA data transfer.
*   You can test this project by connecting and analog 
*   sensor on Pin P0[4] and see the sampled data from Bridge
*   Control Panel. In addition to the I2C read buffer, a 
*   simple write buffer is added to test I2C write operations.
*
*   \author Davide Marzorati
*   \data March 6, 2020
*/

#include "project.h"

// Size of I2C Write an Read Buffer
#define WR_BUFFER_SIZE 5
#define RD_BUFFER_SIZE 32

// I2C Packet Start and End values
#define PACKET_SOP 0xA0
#define PACKET_EOP 0xC0

// I2C Write Packet positions
#define PACKET_SOP_POS  (0x00u) // Start
#define PACKET_CMD_POS  (0x01u) // Command
#define PACKET_DATA_POS (0x02u) // Data
#define PACKET_EOP_POS  (0x04u) // End

// I2C Write Command values
#define PACKET_LED_CMD (0x01u)

// DMA ADC Configuration 
#define DMA_ADC_BYTES_PER_BURST     2
#define DMA_ADC_REQUEST_PER_BURST   1
#define DMA_ADC_SRC_BASE            (CYDEV_PERIPH_BASE)
#define DMA_ADC_DEST_BASE           (CYDEV_SRAM_BASE)

// DMA I2C Configuration 
#define DMA_I2C_BYTES_PER_BURST     16
#define DMA_I2C_REQUEST_PER_BURST   0
#define DMA_I2C_SRC_BASE            (CYDEV_SRAM_BASE)
#define DMA_I2C_DEST_BASE           (CYDEV_SRAM_BASE)

// I2C Read Buffer
uint8_t i2cReadBuffer [RD_BUFFER_SIZE] = {0};
uint8_t i2cWriteBuffer[WR_BUFFER_SIZE];

// ADC Buffer
uint16_t adcBuffer[RD_BUFFER_SIZE/2] = {0};

// Flag for I2C operation check
uint8_t I2CFlag = 0u;


void I2C_Slave_ISR_EntryCallback() {
    // Write 0 to control register to block DMA_I2C drq
    Control_Reg_Write(0);
}

void I2C_Slave_ISR_ExitCallback () {
    // Set I2C flag to 1 when I2C transaction is complete
    I2CFlag = 1;
}

// DMA Configuration function prototypes
void DMA_ADC_Config(void);
void DMA_I2C_Config(void);


int main(void)
{
    // Configure DMA
    DMA_ADC_Config();
    DMA_I2C_Config();
    // Init I2C Read and Write buffers
    I2C_Slave_SlaveInitReadBuf(i2cReadBuffer, RD_BUFFER_SIZE);
    I2C_Slave_SlaveInitWriteBuf(i2cWriteBuffer, WR_BUFFER_SIZE);
    // Start I2C Slave component
    I2C_Slave_Start();
    // Start PWM component
    PWM_Led_Start();
    // Start ADC
    ADC_DelSig_Start();
    ADC_DelSig_StartConvert();
    
    //Control_Reg_Write(0);
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    for(;;)
    {
        // Check if I2C Write or Read operation occurred
        if (I2CFlag != 0)
        {
            // Reset flag variable
            I2CFlag = 0;
            // If write operation occurred
            if (0u != (I2C_Slave_SSTAT_WR_CMPLT & I2C_Slave_SlaveStatus()))
            {
                // If size of data written equal to write buffer size
                if ( I2C_Slave_SlaveGetWriteBufSize() == WR_BUFFER_SIZE)
                {
                    // Check packet start and end values
                    if ((i2cWriteBuffer[PACKET_SOP_POS] == PACKET_SOP) && 
                        (i2cWriteBuffer[PACKET_EOP_POS] == PACKET_EOP)) 
                    {
                        // Check if it is a command for the LED
                        if (i2cWriteBuffer[PACKET_CMD_POS] == PACKET_LED_CMD) 
                        {
                            // Update PWM compare value
                            PWM_Led_WriteCompare(i2cWriteBuffer[PACKET_DATA_POS]);
                        }
                    }
                }
                
                // Reset write buffer 
                I2C_Slave_SlaveClearWriteBuf();
                (void) I2C_Slave_SlaveClearWriteStatus();
            }
            // If read operation occurred
            if (0u != (I2C_Slave_SSTAT_RD_CMPLT & I2C_Slave_SlaveStatus()))
            {
                Control_Reg_Write(1);
                // Reset read buffer 
                I2C_Slave_SlaveClearReadBuf();
                (void) I2C_Slave_SlaveClearReadStatus();
            }
        }
    }
}


void DMA_ADC_Config(void) {
    // DMA_Chan is used to store the DMA Channel
    uint8_t DMA_Chan;
    // DMA_TD array is used to store all of the TDs associated with this channel
    uint8_t DMA_TD[1];
    
    /* DMA Configuration steps */
    
    // DMA Initialiazion done for DMA Channel
    DMA_Chan = DMA_ADC_DmaInitialize(DMA_ADC_BYTES_PER_BURST, DMA_ADC_REQUEST_PER_BURST,
                                HI16(DMA_ADC_SRC_BASE), HI16(DMA_ADC_DEST_BASE));
    // Allocate TD for DMA Channel
    DMA_TD[0] = CyDmaTdAllocate();
    
    // Configure TD[0]
    CyDmaTdSetConfiguration(DMA_TD[0], 32, DMA_TD[0], TD_INC_DST_ADR | TD_SWAP_EN | DMA_ADC__TD_TERMOUT_EN);
    
    // Configure the TD address
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32) ADC_DelSig_DEC_SAMP_PTR),
                                 LO16((uint32) adcBuffer));
    
    // Map TD to DMA Channel
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    
    // Enable channel
    CyDmaChEnable(DMA_Chan, 1);
}

void DMA_I2C_Config(void) {
    // DMA_Chan is used to store the DMA Channel
    uint8_t DMA_Chan;
    // DMA_TD array is used to store all of the TDs associated with this channel
    uint8_t DMA_TD[1];
    
    /* DMA Configuration steps */
    
    // DMA Initialiazion done for DMA Channel
    DMA_Chan = DMA_I2C_DmaInitialize(DMA_I2C_BYTES_PER_BURST, DMA_I2C_REQUEST_PER_BURST,
                                HI16(DMA_I2C_SRC_BASE), HI16(DMA_I2C_DEST_BASE));
    // Allocate TD for DMA Channel
    DMA_TD[0] = CyDmaTdAllocate();
    
    // Configure TD[0]
    CyDmaTdSetConfiguration(DMA_TD[0], 32, DMA_TD[0], TD_INC_SRC_ADR | TD_INC_DST_ADR | DMA_I2C__TD_TERMOUT_EN);
    
    // Configure the TD address
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32) adcBuffer),
                                 LO16((uint32) i2cReadBuffer));
    
    // Map TD to DMA Channel
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    
    // Enable channel
    CyDmaChEnable(DMA_Chan, 1);
}

/* [] END OF FILE */
